﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Operators;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Optimization;
using System.Threading;

namespace HeuristicLab.Problems.MetaOptimization {
  /// <summary>
  /// A base class for operators which evaluate TSP solutions.
  /// </summary>
  [Item("MetaOptimizationEvaluator", "A base class for operators which evaluate Meta Optimization solutions.")]
  [StorableClass]
  public class MetaOptimizationEvaluator : SingleSuccessorOperator, IMetaOptimizationEvaluator {
    private bool algorithmStopped;

    public ILookupParameter<DoubleValue> QualityParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["Quality"]; }
    }

    public ILookupParameter<IAlgorithm> AlgorithmParameter {
      get { return (ILookupParameter<IAlgorithm>)Parameters["Algorithm"]; }
    }
    public ILookupParameter<IParameterSet> ParameterSetParameter {
      get { return (ILookupParameter<IParameterSet>)Parameters["ParameterSet"]; }
    }

    public MetaOptimizationEvaluator()
      : base() {
      Parameters.Add(new LookupParameter<DoubleValue>("Quality", "The evaluated quality of the TSP solution."));
      Parameters.Add(new LookupParameter<IAlgorithm>("Algorithm", "Missing description."));
      Parameters.Add(new LookupParameter<IParameterSet>("ParameterSet", "Missing description."));
    }

    public override IOperation Apply() {
      AlgorithmParameter.ActualValue.Prepare();
      ParametrizeAlgorithm();
      algorithmStopped = false;
      AlgorithmParameter.ActualValue.Stopped += new EventHandler(ActualValue_Stopped);
      AlgorithmParameter.ActualValue.Start();
      while (!algorithmStopped) {
        Thread.Sleep(1000); // wait for algorithm to complete; do not rely on Algorithm.ExecutionState here, because change of ExecutionState happens before Run is added (which causes problems because Algorithm might get cloned when its started already)
      }
      AlgorithmParameter.ActualValue.Stopped -= new EventHandler(ActualValue_Stopped);
      this.QualityParameter.ActualValue = (DoubleValue)AlgorithmParameter.ActualValue.Results["BestQuality"].Value;
      return base.Apply();
    }

    void ActualValue_Stopped(object sender, EventArgs e) {
      algorithmStopped = true;
    }

    private void ParametrizeAlgorithm() {
      foreach (IParameterConfiguration parameter in ParameterSetParameter.ActualValue.Parameters) {
        if (parameter.Category == "Algorithm") {
          this.AlgorithmParameter.ActualValue.Parameters[parameter.Parameter.Name].ActualValue = parameter.Parameter.ActualValue;
        } else if (parameter.Category == "Problem") {
          this.AlgorithmParameter.ActualValue.Problem.Parameters[parameter.Parameter.Name].ActualValue = parameter.Parameter.ActualValue;
        }
      }
    }
  }
}
